#include <stdio.h>
#include <math.h>
#include <windows.h>

#define MYFLOAT double
#define MYDOUBLE double


#define M_PI 3.1415927

#define MAX_SYNTH_FILTER_PARTIALS  1024
#define NMODES 100
#define BUFFERSIZE 220
#define SAMPLINGRATE ((MYDOUBLE)22050)

void synth_filter_float(int nsamples,MYFLOAT *input,MYFLOAT *output,
                        int n_partials,MYDOUBLE *a,MYDOUBLE *cr,MYDOUBLE *ci2,
                        int resetQ,MYDOUBLE deadline);

// THis one is slower by a small amount (5%)
void synth_filter_float2(int nsamples,MYFLOAT *input,MYFLOAT *output,
                        int n_partials,MYDOUBLE *a,MYDOUBLE *R2,MYDOUBLE *twoRCosTheta,
                        int resetQ,MYDOUBLE deadline);

int main(int ac,char *av[]) {
    MYDOUBLE ampl[NMODES],freq[NMODES], damp[NMODES];
    MYDOUBLE cr[NMODES],ci2[NMODES],cc_plus[NMODES],cc_minus[NMODES];
    MYDOUBLE twoRCosTheta[NMODES],R2[NMODES];
    MYFLOAT input[BUFFERSIZE],output[BUFFERSIZE];
    int ntimes = 5000;
    int i,resetQ=0,k;
    
    MYDOUBLE deadline = 0.;
    MYDOUBLE tottime;
    MYDOUBLE real_time =  ntimes * BUFFERSIZE/SAMPLINGRATE;
    DWORD tick_start,tick_end;
    
    /* Have to be careful that values are plausible, otherwise floating
    point under/overflow may mess things uop
    */
    for(i=0;i<NMODES;i++) {        /* faked values */
        ampl[i] = 1.;
        freq[i] = 50*sqrt((double)i);
        damp[i] = freq[i]/10.;
        cr[i] = exp(-damp[i]/SAMPLINGRATE) * cos(2*M_PI*freq[i]/SAMPLINGRATE);
        ci2[i] = exp(-damp[i]/SAMPLINGRATE) * sin(2*M_PI*freq[i]/SAMPLINGRATE);
        ci2[i]  *= ci2[i];
        cc_plus[i] = cr[i] + sqrt(1. - ci2[i]);
        cc_minus[i] = cr[i] - sqrt(1. - ci2[i]);
        R2[i] = exp(-2*damp[i]/SAMPLINGRATE);
        twoRCosTheta[i] = 2*cr[i];
        /*printf("(cr,ci)=(%e, %e)\n",cr[i],ci[i]);    */
    }
    for(k=0;k<BUFFERSIZE;k++) {
        input[k] = ((MYFLOAT) (k*13))/20.;
    }
    tick_start = GetTickCount();
#define RESON_IMP 
#ifdef RESON_IMP
    for(i=0;i<ntimes;i++) {
        synth_filter_float2(BUFFERSIZE,&input[0],&output[0],NMODES,ampl,R2,twoRCosTheta,resetQ,deadline);
    }
#else
    for(i=0;i<ntimes;i++) {
        synth_filter_float(BUFFERSIZE,&input[0],&output[0],NMODES,ampl,cc_plus,cc_minus,resetQ,deadline);
    }
#endif
    tick_end = GetTickCount();
    tottime = (tick_end - tick_start) / 1000.0;
    printf("Runtime: %f sec realtime: %f\n",tottime,real_time);
    printf("Can do %f partials at %f Hz\n",real_time*NMODES/tottime,SAMPLINGRATE);
    return 0;
}


/*
Filter input[] interpreted as interaction force, producing output[].
Algorithm:
output[t] = SUM(n=1:n_partials) REAL(y(n,t));
y(n,t)=twoRCosTheta[n]*y(t-1)-R2[n]*y(t-2)+a[n]*input[t];
If resetQ==true, y(n,-1) and y(n,-2) is taken as zero, otherwise it remembered value
after last call of function.
*/

void synth_filter_float2(int nsamples,MYFLOAT *input,MYFLOAT *output,
                        int n_partials,MYDOUBLE *a,MYDOUBLE *R2,MYDOUBLE *twoRCosTheta,
                        int resetQ,MYDOUBLE deadline)
{
    static MYDOUBLE yt_1[MAX_SYNTH_FILTER_PARTIALS],yt_2[MAX_SYNTH_FILTER_PARTIALS];
    register int k;
    register MYDOUBLE ynew,tmp_yt_1,tmp_yt_2,tmp_twoRCosTheta,tmp_R2,tmp_a;
    int i;
    if(n_partials > MAX_SYNTH_FILTER_PARTIALS) {
        fprintf(stderr,"synth_filter_float() called with too many partials (maximal %d)\n",(int)MAX_SYNTH_FILTER_PARTIALS);
        return;
    }
    /* Init old values of buffers to zero if resetting */
    if(resetQ) {
        for(i=0;i<n_partials;i++) {
            yt_1[i] = yt_2[i] = 0.0;
        }
    }
    ZeroMemory(output,nsamples*sizeof(MYFLOAT));
    
    for(i=0;i<n_partials;i++) {
        tmp_twoRCosTheta = twoRCosTheta[i];
        tmp_R2 = R2[i];
        tmp_a = a[i];
        tmp_yt_1 = yt_1[i];
        tmp_yt_2 = yt_2[i];
        for(k=0;k<nsamples;k++) {
            ynew = tmp_twoRCosTheta * tmp_yt_1 - tmp_R2 * tmp_yt_2 + tmp_a * input[k];
            tmp_yt_2 = tmp_yt_1;
            tmp_yt_1 = ynew;
            output[k] += ynew;
        }
        yt_1[i] = tmp_yt_1;
        yt_2[i] = tmp_yt_2;
    }
}


void synth_filter_float(int nsamples,MYFLOAT *input,MYFLOAT *output,
                        int n_partials,MYDOUBLE *a,MYDOUBLE *cc_plus,MYDOUBLE *cc_minus,
                        int resetQ,MYDOUBLE deadline)
{
    static MYDOUBLE urPrev[MAX_SYNTH_FILTER_PARTIALS],
        uiPrev[MAX_SYNTH_FILTER_PARTIALS];
    int i;
    /* urPrev holds previous values real part  of  y(n,-1), don't need im part */
    register int k;
    register MYDOUBLE out_r_new,out_i_new,out_r_prev;
    register MYDOUBLE tmp_ccminus,tmp_ccplus,tmp_a;
    
    if(n_partials > MAX_SYNTH_FILTER_PARTIALS) {
        fprintf(stderr,"synth_filter_float() called with too many partials (maximal %d)\n",(int)MAX_SYNTH_FILTER_PARTIALS);
        return;
    }
    /* Init old values of buffers to zero if resetting */
    if(resetQ) {
        for(i=0;i<n_partials;i++) {
            uiPrev[i] = urPrev[i] = 0.0;
        }
    }
    ZeroMemory(output,nsamples*sizeof(MYFLOAT));
    
    for(i=0;i<n_partials;i++) {
        out_r_prev = urPrev[i];
        out_i_new = uiPrev[i];
        tmp_ccplus = cc_plus[i];
        tmp_ccminus = cc_minus[i];
        tmp_a = a[i];
        for(k=0;k<nsamples;k++) {
            out_r_new = tmp_ccminus * out_r_prev - out_i_new + tmp_a * input[k];
            out_i_new = tmp_ccplus * out_i_new + out_r_prev;
            out_r_prev = out_r_new;
            output[k] += out_i_new;
            //output[k] += out_r_new; // add this for engine sounds
        }
        urPrev[i] = out_r_new;
        uiPrev[i] = out_i_new;
    }
}
/*
Filter input[] interpreted as interaction force, producing output[].
Algorithm:
output[t] = SUM(n=1:n_partials) REAL(y(n,t));
y(n,t+1) = (cr[n] + sqrt(-1)*ci[n])y(n,t) + a[n]*input[t];

  If resetQ==true, y(n,-1) is taken as zero, otherwise it remembered value
  after last call of function.
  
    output[] is the audio signal. a[] represents the amplitudes corresponding to
    the interaction point. input[] is the force.
    cr[n] = exp(-damping[n]/SR)cos(2*pi*freq[n]/SR)
    ci[n] = exp(-damping[n]/SR)sin(2*pi*freq[n]/SR)
    with  damping[] and freq[] the dampings and frequencies of the modes,
    and SR is the sampling rate in Hz.
    
      Implementation:
      
        y(t+1) = cr * y(t) - w(t) + input(t) * a
        w(t+1) = ci^2 * y(t) + cr * w(t)
        
          Now 
          y = u  + sqrt(1-ci^2) * v
          w = ci^2 * v
          Then
          u(t+1) = (cr - sqrt(1-ci^2)) * u(t) - v(t) + input(t) * a
          v(t+1) = (cr + sqrt(1-ci^2)) * v(t) + u(t)
          Define 
          cc_minus = cr - sqrt(1-ci^2)
          cc_plus = cr + sqrt(1-ci^2)
          
            Define w*Srate = z, and x(t) = om^2 * Srate*a*input(t), then in Srate -> inf limit:
            
              dy/dt + d*y + z(t) = x(t)
              dz/dt + d*z - om^2*y(t) = 0
              
                This reduces to a single 2nd order eq. for z:
                d^2z/dt^2 + 2*d*dz/dt +(om^2 + d^2)*z = x(t)
                or for y:
                d^2y/dt^2 + 2*d*dy/dt +(om^2 + d^2)*y = d*x(t) + dx/dt
                
                  Sometimes is better to use y for audio for car sound especially
                  
                    Must have n_partials <= MAX_SYNTH_FILTER_PARTIALS
                    
*/
