/**
   A  force model  for combustion  engine. As  superclass,  but misfires
   occasionally.
   Copyright The University of British Columbia, Canada
   @author Kees van den Doel (kvdoel@cs.ubc.ca)
*/
public class CombustionEngineForce extends LoopForce {
    /** Probablility of misfiring per call to {@link #getForce}. */
    private float misfireProb=0.f; 

    /** Constructor. Loads looped audio from named file.
        @param srate sampling rate in Hertz.
        @param fn name of audio file to load data from.
    */
    public CombustionEngineForce(float srate,String fn) {
        super(fn);
    }

    /** Set probability of misfiring per call to {@link #getForce}.
        @param val probability.
    */
    public void setBeatupness(float val) {
        misfireProb = val;
    }

    /** Create force as in superclass but misfire occasionally.
        @param output user provided buffer for force returned.
        @param nsamples number of samples written to output buffer.
    */
    public void getForce(float [] output, int nsamples) {
        if(Math.random() < misfireProb) {
            System.out.println("<*misfired*>");
            // Skip half a sample forward
            ix += this.afBuffer.bufsz/2;
            if(ix >= this.afBuffer.bufsz) {
                ix -= this.afBuffer.bufsz;
            }
        } else {
            super.getForce(output,nsamples);
        }
    }

}


