import java.lang.*;
import java.util.*;

/** Shows how many modes can be synthesized on your machine.
    Copyright The University of British Columbia, Canada
    @author Kees van den Doel (kvdoel@cs.ubc.ca)
 */
public class DemoModalBenchMark {
    static final int NMODES = 100;
    static final int BUFFERSIZE= 128;
    static final float SAMPLINGRATE = 22050f;
    static final float input_gain = 1.0f;
    static final float volume = 1.0f;
    static final int ntimes = 5000; // this many buffers to time
    
    static public void main(String args[]) {
        float[] ampl = new float[NMODES];
    	float[] freq = new float[NMODES];
    	float[] damp = new float[NMODES];
        float[] input = new float[BUFFERSIZE];
        float[]output = new float[BUFFERSIZE];

        float deadline = 0.f;
        float tottime;
        float real_time =  ntimes * BUFFERSIZE/SAMPLINGRATE;

        ModalSonicObject sob = new ModalSonicObject((float)SAMPLINGRATE,NMODES,1,BUFFERSIZE,0);
        
        for(int i=0;i<NMODES;i++) {        /* faked values */
           	sob.modalModel.a[0][i] = 1.f;
           	sob.modalModel.f[i] = (float)(50f*Math.sqrt((float)i));
           	sob.modalModel.d[i] = freq[i]/10.f;
        }
        sob.computeFilter();
        sob.setAGC(false);
        for(int i=0;i<BUFFERSIZE;i++) {
       	    input[i] = (i*13)/20.f;
        }
        // Create simple noop force
        sob.setAudioForce(new AudioForce() {
            public void getForce(float[] out,int nsamples) {
            }
        });
        Date t1 = new Date();
        for(int i=0;i<ntimes;i++) {
            sob.getForce(output,BUFFERSIZE);
            //sob.computeSoundBuffer(output,input,BUFFERSIZE);
        }
        Date t2 = new Date();
        tottime = (t2.getTime() - t1.getTime())/1000.f;
    	System.out.println("Runtime: "+ tottime + " Realtime passed: " +real_time);
        System.out.println("Can do "+real_time*NMODES/tottime + " partials at " + SAMPLINGRATE + " Hz");	
	}

}
